% NLMAIN    Ng and Li's Test-Problem Generator Main Program
%             (for internal use only)



function NL = nglimain(NL)


% Read random numbers form file
%
AllRand = read_rand(NL);


% In theory, a(i) > 0 
% In practice, we generate a(i) in [aLB, aUB]
%
NL.a = NL.aLB + (NL.aUB-NL.aLB) .* AllRand(1:NL.n);


% In theory, p(i) in real
% In practice, we generate p(i) in [pLB, pUB]
%
NL.p = NL.pLB + (NL.pUB-NL.pLB) .* AllRand(NL.n+1:2*NL.n);


% In theory, q(i) such that q(i) < p(i)^2
% In practice, we generate q(i) in [qLB, qUB], where qUB = qPer*(p.^2)
%
NL.qUB = NL.qPer * (NL.p.^2);
NL.q   = NL.qLB + (NL.qUB-NL.qLB) .* AllRand(2*NL.n+1:3*NL.n);


% Calculate r(i) = sqrt( p(i)^2-q(i) )
%
NL.r = sqrt(NL.p.^2 - NL.q);


% For difficult subproblems, we generate alpha(i) such that
%   alpha(i) in [-p(i)+(-2+0.5*LEN)*r(i), -p(i)+(-2+0.975*LEN)*r(i)]  OR
%   alpha(i) in [-p(i)+(sqrt(3)+0.025*LEN)*r(i), -p(i)+(sqrt(3)+0.5*LEN)*r(i)]
% For easy subproblems, we generate alpha(i) such that
%   alpha(i) in [-p(i)+(-2+0.025*LEN)*r(i), -p(i)+(-2+0.5*LEN)*r(i)]  OR
%   alpha(i) in [-p(i)+(sqrt(3)+0.5*LEN)*r(i), -p(i)+(sqrt(3)+0.975*LEN)*r(i)]
%
LEN = 2-sqrt(3);
NL.alpha  = zeros(NL.n,1);	% Initialize alpha
for I = 1:NL.nDiff
  RAND = AllRand(3*NL.n+I);	% rand in [0,1]
  if RAND < 0.5
    RAND = 2*RAND;		% rand in [0,1]
    NL.alpha(I) = ...
      -NL.p(I) + (-2+(1+NL.alphaPer*RAND)*LEN/2) * NL.r(I);
  else
    RAND = 2*RAND-1;		% rand in [0,1]
    NL.alpha(I) = ...
      -NL.p(I) + (sqrt(3)+(1-NL.alphaPer+NL.alphaPer*RAND)*LEN/2) * NL.r(I);
  end
end
for I = NL.nDiff+1:NL.n
  RAND = AllRand(3*NL.n+I);	% rand in [0,1]
  if RAND < 0.5
    RAND = 2*RAND;		% rand in [0,1]
    NL.alpha(I) = ...
      -NL.p(I) + (-2+(1-NL.alphaPer+NL.alphaPer*RAND)*LEN/2) * NL.r(I);
  else
    RAND = 2*RAND-1;		% rand in [0,1]
    NL.alpha(I) = ...
      -NL.p(I) + (sqrt(3)+(1+NL.alphaPer*RAND)*LEN/2) * NL.r(I);
  end
end


% Calculate s(i)
%
NL.s = -4*NL.alpha .* (NL.alpha.^2 + 3*NL.p.*NL.alpha + 3*NL.q);


% Calculate fStar
%
NL.fStar = sum(NL.a .* (NL.alpha.^4 + 4 * NL.p .* NL.alpha.^3 + 6 * NL.q .* NL.alpha.^2 + NL.s .* NL.alpha));




% START ALGORITHM 2




% In theory, d(i) > 0
% In practice, we generate d(i) in [dLB, dUB]
%
NL.d = NL.dLB + (NL.dUB-NL.dLB) .* AllRand(4*NL.n+1:5*NL.n);


% invD = inv(D) = inv(diag(d))
%
NL.invD = diag(1 ./ NL.d);


% In theory, v(i)>=0 and v'*v = sum(v(i)^2) = 1
% In practice, we first generate v(i) in [0,1]
% We then set v(i) := v(i) / sqrt(sum(v(i)^2)), then sum(v(i)^2) = 1
%
NL.v = AllRand(5*NL.n+1:6*NL.n);
NL.v = NL.v / sqrt(sum(NL.v.^2));


% Calculate H = I - 2* v * v'
%
NL.H = eye(NL.n) - 2 * (NL.v * NL.v');


% Calculate yStar = H * inv(D) * alpha
%
NL.yStar = NL.H * NL.invD * NL.alpha;



if isempty(NL.wLB)
  NL.wStar = NL.yStar;

else
  % START ALGORITHM 3

  % In theory, deltaL(i) > 0 and deltaU(i) > 0
  % In practice, we generate deltaL and deltaU in [deltaLB, deltaUB]
  %
  NL.deltaL = NL.deltaLB + (NL.deltaUB-NL.deltaLB) .* AllRand(6*NL.n+1:7*NL.n);
  NL.deltaU = NL.deltaLB + (NL.deltaUB-NL.deltaLB) .* AllRand(7*NL.n+1:8*NL.n);


  % Calc xLB(i) and xUB(i)
  %
  NL.xLB = -NL.p - 2 * NL.r .* (1 + 2 * NL.deltaL);
  NL.xUB = -NL.p + 2 * NL.r .* (1 + 2 * NL.deltaU);


  % Calc yLB & yUB
  %
  NL.yLB      = zeros(NL.n,1);
  NL.yUB      = zeros(NL.n,1);
  v_xLB_d     = NL.v .* NL.xLB ./ NL.d;
  v_xUB_d     = NL.v .* NL.xUB ./ NL.d;
  sum_v_xLB_d = sum(v_xLB_d);
  sum_v_xUB_d = sum(v_xUB_d);
  for I = 1:NL.n
    if NL.v(I)^2 >= 0.5
      NL.yLB(I) = (1-2*NL.v(I)^2)/NL.d(I)*NL.xUB(I) - 2*NL.v(I)*(sum_v_xUB_d - NL.v(I)*NL.xUB(I)/NL.d(I));
      NL.yUB(I) = (1-2*NL.v(I)^2)/NL.d(I)*NL.xLB(I) - 2*NL.v(I)*(sum_v_xLB_d - NL.v(I)*NL.xLB(I)/NL.d(I));
    else
      NL.yLB(I) = (1-2*NL.v(I)^2)/NL.d(I)*NL.xLB(I) - 2*NL.v(I)*(sum_v_xUB_d - NL.v(I)*NL.xUB(I)/NL.d(I));
                                                 NL.yUB(I) = (1-2*NL.v(I)^2)/NL.d(I)*NL.xUB(I) - 2*NL.v(I)*(sum_v_xLB_d - NL.v(I)*NL.xLB(I)/NL.d(I));
    end
  end



  % START ALGORITHM 4

  % Calc k
  %
  NL.k    = (NL.yUB-NL.yLB)./(NL.wUB-NL.wLB);
  NL.invK = diag(1 ./ NL.k);


  % Calc wStar
  %
  NL.wStar = NL.invK * (NL.yStar - NL.yLB) + NL.wLB;
end



write_global(NL);
write_matlab(NL);
write_gams(NL);





function AllRand = read_rand(NL)


fid = fopen(['ngli', num2str(NL.id,'%03u'), '.rn'], 'r');
if fid > 0
  AllRand = fscanf(fid, '%e', [1 inf])';
  fclose(fid);
else
  fprintf('\n');
  error(['The file "ngli', num2str(NL.id,'%03u'), '.rn" does not exist.']); 
end





function write_global(NL)


fid = fopen(['ngli', num2str(NL.id,'%03u'), '.txt'], 'w');

if NL.id <= 300
  fprintf(fid, 'Ng and Li standard test-problem number %d\n\n', NL.id);
else
  fprintf(fid, 'Ng and Li user-defined test-problem number %d\n\n', NL.id);
  fprintf(fid, ['Random numbers are generated and stored in "ngli',...
               num2str(NL.id,'%03u'), '.rn"\n\n']);
end

fprintf(fid, 'Global minimum value:\n');
fprintf(fid, '% 19.15e\n\n', NL.fStar);
fprintf(fid, 'Global minimizer:\n');
for I=1:NL.n
  fprintf(fid, '% 19.15e\n', NL.wStar(I));
end

fclose(fid);





function write_matlab(NL)


fid = fopen(['ngli', num2str(NL.id,'%03u'), '.m'], 'w');

if NL.id <= 300
  fprintf(fid, ['%% Ng and Li standard test-problem number %d ',...
                '(n=%d, level=%d)\n'], NL.id, NL.n, NL.level);
else
  fprintf(fid, '%% Ng and Li user-defined test-problem number %d\n', NL.id);
end
fprintf(fid, '%%\n');
fprintf(fid, '%% This file stores the parameters of the named test-problem\n');
fprintf(fid, '%% for use with the function NLOBJ\n');
fprintf(fid, '%%\n');
if NL.id <= 300
  fprintf(fid, ...
    ['%% The global minimizer and its value ', ...
     'are stored in "ngli', num2str(NL.id,'%03u'), '.txt"\n\n\n']);
else
  fprintf(fid, ...
    '%% The parameters, the global minimizer and its value are stored\n');
  fprintf(fid, ...
    ['%% in "ngli', num2str(NL.id,'%03u'), '.txt"\n\n\n']);
end

fprintf(fid, ['function NL = ngli', num2str(NL.id,'%03u'), '\n\n']);

fprintf(fid, 'NL.n = %d;\n\n', NL.n);

fprintf(fid, 'NL.a = [\n');
for I=1:NL.n
  fprintf(fid, '% 19.15e\n', NL.a(I));
end
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.p = [\n');
for I=1:NL.n
  fprintf(fid, '% 19.15e\n', NL.p(I));
end
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.q = [\n');
for I=1:NL.n
  fprintf(fid, '% 19.15e\n', NL.q(I));
end
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.s = [\n');
for I=1:NL.n
  fprintf(fid, '% 19.15e\n', NL.s(I));
end
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.d = [\n');
for I=1:NL.n
  fprintf(fid, '% 19.15e\n', NL.d(I));
end
fprintf(fid, '];\n\n');

fprintf(fid, 'NL.v = [\n');
for I=1:NL.n
  fprintf(fid, '% 19.15e\n', NL.v(I));
end
fprintf(fid, '];\n\n');


if isempty(NL.wLB)
  fprintf(fid, 'NL.wLB = [];\n\n');
  fprintf(fid, 'NL.yLB = [];\n\n');
  fprintf(fid, 'NL.k   = [];\n\n');
else
  fprintf(fid, 'NL.wLB = % 19.15e;\n\n', NL.wLB);

  fprintf(fid, 'NL.yLB = [\n');
  for I=1:NL.n
    fprintf(fid, '% 19.15e\n', NL.yLB(I));
  end
  fprintf(fid, '];\n\n');

  fprintf(fid, 'NL.k = [\n');
  for I=1:NL.n
    fprintf(fid, '% 19.15e\n', NL.k(I));
  end
  fprintf(fid, '];\n\n');
end


fprintf(fid, 'NL.val = % 19.15e;\n\n', NL.fStar);

fclose(fid);





function write_gams(NL)


fid = fopen(['ngli', num2str(NL.id,'%03u'), '.gms'], 'w');


% Print TITLE statement to file
%
if NL.id <= 300
  fprintf(fid, ...
    ['$TITLE  Ng and Li standard test-problem number %d ',...
     '(n=%d, level=%d)\n\n\n'], NL.id, NL.n, NL.level);
  fprintf(fid, ...
    ['* The global minimizer and its value ',...
        'are stored in "ngli', num2str(NL.id,'%03u'), '.txt"\n\n\n']);
else
  fprintf(fid, ...
    '$TITLE  Ng and Li user-defined test-problem number %d\n\n\n', NL.id);
  fprintf(fid, ...
    '* The parameters, the global minimizer and its value are stored\n');
  fprintf(fid, ...
    ['* in "ngli', num2str(NL.id,'%03u'), '.txt"\n\n\n']);
end


% Print FILE to file
%
fprintf(fid, 'FILE\n');
fprintf(fid, ['res \t /ngli', '%03u', 'gms.txt/;\n'], NL.id);
fprintf(fid, 'PUT \t res;\n\n\n');


% Print SETS to file
%
fprintf(fid, 'SETS\n');
fprintf(fid, ['i \t /1*', '%d', '/;\n'], NL.n);
fprintf(fid, 'ALIAS(i,j);\n\n\n');


% Print SCALAR to file
%
if ~isempty(NL.wLB)
  fprintf(fid, 'SCALAR\n');
  fprintf(fid, 'wLB\n');
  fprintf(fid, '/ \t\t % 19.15e/\n', NL.wLB);
  fprintf(fid, 'wUB\n');
  fprintf(fid, '/ \t\t % 19.15e/;\n\n\n', NL.wUB);
end


% Print PARAMETERS to file
%
fprintf(fid, 'PARAMETERS\n');

fprintf(fid, 'a(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.a(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.a(NL.n));

fprintf(fid, 'p(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.p(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.p(NL.n));

fprintf(fid, 'q(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.q(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.q(NL.n));

fprintf(fid, 's(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.s(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.s(NL.n));

fprintf(fid, 'd(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.d(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.d(NL.n));

fprintf(fid, 'v(i)\n');
fprintf(fid, '/');
for I=1:NL.n-1
  fprintf(fid, '\t %d \t % 19.15e\n', I, NL.v(I));
end
fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.v(NL.n));


if ~isempty(NL.wLB)
  fprintf(fid, 'yLB(i)\n');
  fprintf(fid, '/');
  for I=1:NL.n-1
    fprintf(fid, '\t %d \t % 19.15e\n', I, NL.yLB(I));
  end
  fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.yLB(NL.n));

  fprintf(fid, 'k(i)\n');
  fprintf(fid, '/');
  for I=1:NL.n-1
    fprintf(fid, '\t %d \t % 19.15e\n', I, NL.k(I));
  end
  fprintf(fid, '\t %d \t % 19.15e/\n', NL.n, NL.k(NL.n));
end


fprintf(fid, 'H(i,j);\n\n');

fprintf(fid, 'LOOP(i, LOOP(j,\n');
fprintf(fid, 'H(i,j) = -2*v(i)*v(j) + 1 $ (ORD(i) EQ ORD(j));\n');
fprintf(fid, '); );\n\n\n');


% Print VARIABLES to file
%
fprintf(fid, 'VARIABLES\n');
fprintf(fid, 'w(i)\n');
if ~isempty(NL.wLB)
  fprintf(fid, 'y(i)\n');
end
fprintf(fid, 'x(i)\n');
fprintf(fid, 'obj;\n\n');

if isempty(NL.wLB)
  fprintf(fid, '\n');
else
  fprintf(fid, 'LOOP(i,\n');
  fprintf(fid, 'w.lo(i) =\t wLB;\n');
  fprintf(fid, 'w.up(i) =\t wUB;\n');
  fprintf(fid, ');\n\n');

  for I=1:NL.n
    fprintf(fid, 'y.lo(''%d'') =\t % 19.15e;\n', I, NL.yLB(I));
    fprintf(fid, 'y.up(''%d'') =\t % 19.15e;\n', I, NL.yUB(I));
  end
  fprintf(fid, '\n');

  for I=1:NL.n
    % GAMS cannot handle both xLB<0 and xUB<0
    if NL.xUB(I) > 0
      fprintf(fid, 'x.lo(''%d'') =\t % 19.15e;\n', I, NL.xLB(I));
      fprintf(fid, 'x.up(''%d'') =\t % 19.15e;\n', I, NL.xUB(I));
    end
  end
  fprintf(fid, '\n\n');
end


% Print EQUATIONS to file
%
fprintf(fid, 'EQUATIONS\n');
if ~isempty(NL.wLB)
  fprintf(fid, 'tmp_y(i)\n');
end
fprintf(fid, 'tmp_x(i)\n');
fprintf(fid, 'tmp_obj;\n\n');

if ~isempty(NL.wLB)
  %fprintf(fid, 'tmp_y(i)..\n');
  %fprintf(fid, 'y(i) =e= k(i)*(w(i)-wLB(i))+yLB(i);\n\n');
  fprintf(fid, 'tmp_y(i)..\n');
  fprintf(fid, 'y(i) =e= k(i)*(w(i)-wLB) + yLB(i);\n\n');
end

if isempty(NL.wLB)
  fprintf(fid, 'tmp_x(i)..\n');
  fprintf(fid, 'x(i) =e= d(i) * sum(j, H(i,j)*w(j));\n\n');
else
  fprintf(fid, 'tmp_x(i)..\n');
  fprintf(fid, 'x(i) =e= d(i) * sum(j, H(i,j)*y(j));\n\n');
end

% Updated in version 1.1
%fprintf(fid, 'tmp_obj..\n');
%fprintf(fid, ['obj =e= sum(i, ', ...
%'a(i)*(x(i)**4+4*p(i)*x(i)**3+6*q(i)*x(i)**2+s(i)*x(i)));\n\n\n']);
fprintf(fid, 'tmp_obj..\n');
fprintf(fid, ['obj =e= sum(i, ', ...
'a(i)*(power(x(i),4)+4*p(i)*power(x(i),3)+6*q(i)*sqr(x(i))+s(i)*x(i)));\n\n\n']);


% Print MODEL and SOLVE to file
%
fprintf(fid, 'MODEL testproblem /ALL/;\n');
fprintf(fid, 'testproblem.optfile   = 1;\n');
fprintf(fid, 'testproblem.workspace = 1024;\n');
fprintf(fid, 'SOLVE testproblem USING NLP MINIMIZING obj;\n\n\n');


% Print results to file
%
fprintf(fid, 'PUT "Minimum value:"/;\n');
fprintf(fid, 'PUT obj.l:22:15/;\n\n');
fprintf(fid, 'PUT "Minimizer:"/;\n');
fprintf(fid, 'LOOP(i, PUT w.l(i):22:15/);\n');


fclose(fid);
