% NGLISTD    Ng and Li's Standard Test-Problems Generator.
%
% NGLISTD generates a set of 300 standard test-problems, namely,
%   ngli001.m   -- ngli300.m   (for Matlab),
%   ngli001.gms -- ngli300.gms (for GAMS),
%   ngli001.txt -- ngli300.txt (global minimizer and its value).
% This standard set includes 10 different sizes of problems:
%   n = 2, 5, 10, 20, 50, 100, 200, 500, 1000 and 2000.
% For each size of the problems, except n=2, it includes 6 levels of
% difficulties:  level = 0, 1, 2, 3, 4, and 5;
% and each level consists of 5 instants of test-problems.
% For n=2, it includes 3 levels of difficulties:  L = 0, 1, and 2;
% and each level consists of 10 instants of test-problems.
%
% NGLISTD(ALL_ID) generates the standard test-problems with IDs listed
% in the array ALL_ID.  For example, nglistd(61:90) generates the 
% 30 standard test-problems with n=10.
%
% The following is a list of the 300 standard test-problems:
% NAMES 		   n	level	  NAMES			   n	level
% ngli001 -- ngli010	   2	  0
% ngli011 -- ngli020	   2	  1
% ngli021 -- ngli030	   2	  2
% ngli031 -- ngli035	   5	  0	  ngli036 -- ngli040	   5	  1
% ngli041 -- ngli045	   5	  2	  ngli046 -- ngli050	   5	  3
% ngli051 -- ngli055	   5	  4	  ngli056 -- ngli060	   5	  5
% ngli061 -- ngli065	  10	  0	  ngli066 -- ngli070	  10	  1
% ngli071 -- ngli075	  10	  2	  ngli076 -- ngli080	  10	  3
% ngli081 -- ngli085	  10	  4	  ngli086 -- ngli090	  10	  5
% ngli091 -- ngli095	  20	  0	  ngli096 -- ngli100	  20	  1
% ngli101 -- ngli105	  20	  2	  ngli106 -- ngli110	  20	  3
% ngli111 -- ngli115	  20	  4	  ngli116 -- ngli120	  20	  5
% ngli121 -- ngli125	  50	  0	  ngli126 -- ngli130	  50	  1
% ngli131 -- ngli135	  50	  2	  ngli136 -- ngli140	  50	  3
% ngli141 -- ngli145	  50	  4	  ngli146 -- ngli150	  50	  5
% ngli151 -- ngli155	 100	  0	  ngli156 -- ngli160	 100	  1
% ngli161 -- ngli165	 100	  2	  ngli166 -- ngli170	 100	  3
% ngli171 -- ngli175	 100	  4	  ngli176 -- ngli180	 100	  5
% ngli181 -- ngli185	 200	  0	  ngli186 -- ngli190	 200	  1
% ngli191 -- ngli195	 200	  2	  ngli196 -- ngli200	 200	  3
% ngli201 -- ngli205	 200	  4	  ngli206 -- ngli210	 200	  5
% ngli211 -- ngli215	 500	  0	  ngli216 -- ngli220	 500	  1
% ngli221 -- ngli225	 500	  2	  ngli226 -- ngli230	 500	  3
% ngli231 -- ngli235	 500	  4	  ngli236 -- ngli240	 500	  5
% ngli241 -- ngli245	1000	  0	  ngli246 -- ngli250	1000	  1
% ngli251 -- ngli255	1000	  2	  ngli256 -- ngli260	1000	  3
% ngli261 -- ngli265	1000	  4	  ngli266 -- ngli270	1000	  5
% ngli271 -- ngli275	2000	  0	  ngli276 -- ngli280	2000	  1
% ngli281 -- ngli285	2000	  2	  ngli286 -- ngli290	2000	  3
% ngli291 -- ngli295	2000	  4	  ngli296 -- ngli300	2000	  5



function nglistd(AllID)



if nargin == 0
  AllID = 1:300;
elseif any(AllID > 300)
  error('Invalid standard test-problem IDs.');
end


NL.aLB = 0.1;
NL.aUB = 1.0;

NL.pLB = -10;
NL.pUB =  10;

NL.qLB  = -100;
NL.qPer = 0.95;

NL.alphaPer = 0.95;

NL.dLB = 0.1;
NL.dUB = 1.0;

NL.deltaLB = 0.2;
NL.deltaUB = 0.5;

NL.wLB = -10;
NL.wUB =  10;



for ID = AllID
  NL.id    = ID;
  NL.class = floor((ID-1)/30);

  switch NL.class
  case 0
    NL.n = 2;
  case 1
    NL.n = 5;
  case 2
    NL.n = 10;
  case 3
    NL.n = 20;
  case 4
    NL.n = 50;
  case 5
    NL.n = 100;
  case 6
    NL.n = 200;
  case 7
    NL.n = 500;
  case 8
    NL.n = 1000;
  case 9
    NL.n = 2000;
  end

  if NL.class == 0
    NL.level = floor((ID-1)/10);		% level = 0:2
    NL.nDiff = NL.level;
  else
    NL.level = floor((ID-1-NL.class*30)/5);	% level = 0:5
    NL.nDiff = round(NL.n * NL.level/5);
  end

  fprintf(['Generating test-problem number %03d ', ...
           '(n=%04d, instant=%02d)... '], ...
          NL.id, NL.n, mod(NL.id-1,30)+1);
  NL = nglimain(NL);
  fprintf('Done.\n');
end
